/*
 * ----------------------------------------------
 *
 * This software belongs to the zzzutils package.
 *
 * Copyright (c) 2003 Fabian "zzznowman" Pietsch.
 * All rights reserved.
 *
 * ----------------------------------------------
 */

/*
 * zbtree.h -- provide an unbalanced b-tree implementation for zzzutils
 *
 **********************************************************************
 *
 * brief ChangeLog:
 *   2003-08-06 to -08 first implementation inside of zzzupdate-xmms.c
 *   2003-08-08 abstraction as zbtree.{c,h}
 *
 **********************************************************************
 *
 *
 */


/* zbtree_enum.stack incrementation step size
 * zbtree_next() will enlarge/shrink its enumeration stack
 * by this amount of elements at a time
 */
#define ZBTREE_SINC 4


/* zbtree base structure */
struct zbtree {
	char *v;		/* value		*/
	struct zbtree *n_le;	/* next, less or equal	*/
	struct zbtree *n_gt;	/* next, greater	*/
};

/* zbtree_next() enumeration status preservation */
struct zbtree_enum {
	int stack_depth;	/* elements currently on stack	*/
	int stack_size;		/* elements fitting on stack	*/
	struct zbtree **stack;	/* the element stack itself	*/
};

#if defined(DEBUG)
/* zbtree_dumpenum -- zbtree_enum corruption tracking */
void zbtree_dumpenum(struct zbtree_enum *e)
{
	int i;
	
	zwarn("zbtree_dumpenum(): ===begin=== for enum at %p", e);
	zwarn("zbtree_dumpenum(): stack_depth == %i, stack_size == %i", e->stack_depth, e->stack_size);

	/* check for general stack validness */
	if (e->stack_depth > e->stack_size)
		zerror("zbtree_dumpenum(): !!! stack_depth > stack_size !!!");
	else {
		zwarn("zbtree_dumpenum(): stack dump:");
		/* loop through e's stack */
		for (i = 0; i < e->stack_depth; i++)
		{
			zwarn("zbtree_dumpenum(): e->stack[%i] == %p", i, e->stack[i]);
		}
	}
	
	zwarn("zbtree_dumpenum(): === end === for enum at %p", e);
}
#endif


/* prototypes for zbtree.c */

/* zbtree_add -- add a new element to a struct zbtree chain */
struct zbtree *zbtree_add(struct zbtree **bt, char *v);

/* zbtree_free -- free an entire struct zbtree chain */
void zbtree_free(struct zbtree **bt, const int freev);

/* zbtree_next -- allow enumeration through zbtree elements
 * Attention: e->stack _must_ be NULL if zbtree_next(..., e) never called before!
 */
struct zbtree *zbtree_next(struct zbtree *bt, struct zbtree_enum *e);
